#ifndef INCLUDE_VMINST_H__
#define INCLUDE_VMINST_H__

#include <stdint.h>

#include "scythe/object.h"
#include "scythe/config.h"
#include "scythe/impl_utils.h"

typedef uintptr_t scm_opcode_t;

struct scm_vm_inst_opd_noopd     { };
struct scm_vm_inst_opd_obj       { ScmObj o1; };
struct scm_vm_inst_opd_obj_obj   { ScmObj o1; ScmObj o2; };
struct scm_vm_inst_opd_si        { int o1; };
struct scm_vm_inst_opd_si_si     { int o1; int o2; };
struct scm_vm_inst_opd_si_si_obj { int o1; int o2; ScmObj o3; };
struct scm_vm_inst_opd_iof       { int o1; };

union scm_vm_inst_operands {
  struct scm_vm_inst_opd_noopd noopd;
  struct scm_vm_inst_opd_obj obj;
  struct scm_vm_inst_opd_obj_obj obj_obj;
  struct scm_vm_inst_opd_si si;
  struct scm_vm_inst_opd_si_si si_si;
  struct scm_vm_inst_opd_si_si_obj si_si_obj;
  struct scm_vm_inst_opd_iof iof;
};

#define SCM_VM_INST_OPERANDS_ALIGN SCM_ALIGNOF(union scm_vm_inst_operands)

struct scm_vm_inst_noopd {
  scm_opcode_t op;
  struct scm_vm_inst_opd_noopd opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

struct scm_vm_inst_obj {
  scm_opcode_t op;
  struct scm_vm_inst_opd_obj opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

struct scm_vm_inst_obj_obj {
  scm_opcode_t op;
  struct scm_vm_inst_opd_obj_obj opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

struct scm_vm_inst_si {
  scm_opcode_t op;
  struct scm_vm_inst_opd_si opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

struct scm_vm_inst_si_si {
  scm_opcode_t op;
  struct scm_vm_inst_opd_si_si opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

struct scm_vm_inst_si_si_obj {
  scm_opcode_t op;
  struct scm_vm_inst_opd_si_si_obj opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

struct scm_vm_inst_iof {
  scm_opcode_t op;
  struct scm_vm_inst_opd_iof opd
    __attribute((aligned(SCM_VM_INST_OPERANDS_ALIGN)));
};

enum {
  SCM_OPCODE_NOP = 0x0000,        /*  0: no operation */
  SCM_OPCODE_HALT,                /*  1: stop calculation */
  SCM_OPCODE_INT,                 /*  2: interrupt */
  SCM_OPCODE_CFRAME,              /*  3; create a continuation frame */
  SCM_OPCODE_EFRAME,              /*  4; create a environment frame */
  SCM_OPCODE_EPOP,                /*  5; pop a environment frame */
  SCM_OPCODE_ESHIFT,              /*  6; shift environment frames */
  SCM_OPCODE_IMMVAL,              /*  7: copy immediate value to val register */
  SCM_OPCODE_PUSH,                /*  8: push value of val register */
  SCM_OPCODE_MVPUSH,              /*  9: push value of val register */
  SCM_OPCODE_RETURN,              /* 10: return from function */
  SCM_OPCODE_PCALL,               /* 11: primitive function call */
  SCM_OPCODE_CALL,                /* 12: function call */
  SCM_OPCODE_TAIL_CALL,           /* 13: function tail call */
  SCM_OPCODE_GREF,                /* 14: refere global variable */
  SCM_OPCODE_GDEF,                /* 15: define global variable */
  SCM_OPCODE_GSET,                /* 16: update global variable */
  SCM_OPCODE_LREF,                /* 17: refere local variable */
  SCM_OPCODE_LSET,                /* 18: update local variable */
  SCM_OPCODE_JMP,                 /* 19: jump */
  SCM_OPCODE_JMPT,                /* 20: jump if true */
  SCM_OPCODE_JMPF,                /* 21: jump if false */
  SCM_OPCODE_LBOX,                /* 22: box local variable */
  SCM_OPCODE_LBREF,               /* 23: refer boxed local variable */
  SCM_OPCODE_LBSET,               /* 24: update boxed local variable */
  SCM_OPCODE_CLOSE,               /* 25: make closure */
  SCM_OPCODE_DEMINE,              /* 26: demine variable */
  SCM_OPCODE_EMINE,               /* 27: make enviroment frame */
                                  /*     and make it mine field */
  SCM_OPCODE_EDEMINE,             /* 28: demine enviromnet frame with */
                                  /*     incomplete enviromnet frame as */
                                  /*     initial value */
  SCM_OPCODE_MRVC,                /* 29: Multiple-Return-Value Check */
  SCM_OPCODE_MRVE,                /* 30: Multiple-Return-Value Error */
  SCM_OPCODE_MODULE,              /* 31: find a module  */

  SCM_VM_INST_NR_OP,
};

enum {
  SCM_VM_INST_FMT_NOOPD = 0,          /* OPCODE */
  SCM_VM_INST_FMT_OBJ,                /* OPCODE || ScmObj */
  SCM_VM_INST_FMT_OBJ_OBJ,            /* OPCODE || ScmObj || ScmObj */
  SCM_VM_INST_FMT_SI,                 /* OPCODE || INT */
  SCM_VM_INST_FMT_SI_SI,              /* OPCODE || INT || INT */
  SCM_VM_INST_FMT_SI_SI_OBJ,          /* OPCODE || INT || INT || ScmObj */
  SCM_VM_INST_FMT_IOF,                /* OPCODE || OFFSET(INT) */

  SCM_VM_INST_NR_FMT
};

enum {
  SCM_VM_INST_SZ_OP        = sizeof(scm_opcode_t),
  SCM_VM_INST_SZ_NOOPD     = sizeof(struct scm_vm_inst_noopd),
  SCM_VM_INST_SZ_OBJ       = sizeof(struct scm_vm_inst_obj),
  SCM_VM_INST_SZ_OBJ_OBJ   = sizeof(struct scm_vm_inst_obj_obj),
  SCM_VM_INST_SZ_SI        = sizeof(struct scm_vm_inst_si),
  SCM_VM_INST_SZ_SI_SI     = sizeof(struct scm_vm_inst_si_si),
  SCM_VM_INST_SZ_SI_SI_OBJ = sizeof(struct scm_vm_inst_si_si_obj),
  SCM_VM_INST_SZ_IOF       = sizeof(struct scm_vm_inst_iof),
};

enum {
  SCM_VM_INST_SZ_NOP       = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_HALT      = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_INT       = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_UNDEF     = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_UNINIT    = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_CFRAME    = SCM_VM_INST_SZ_IOF,
  SCM_VM_INST_SZ_EFRAME    = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_EPOP      = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_ESHIFT    = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_IMMVAL    = SCM_VM_INST_SZ_OBJ,
  SCM_VM_INST_SZ_PUSH      = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_MVPUSH    = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_RETURN    = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_PCALL     = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_CALL      = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_TAIL_CALL = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_GREF      = SCM_VM_INST_SZ_OBJ_OBJ,
  SCM_VM_INST_SZ_GDEF      = SCM_VM_INST_SZ_OBJ_OBJ,
  SCM_VM_INST_SZ_GSET      = SCM_VM_INST_SZ_OBJ_OBJ,
  SCM_VM_INST_SZ_LREF      = SCM_VM_INST_SZ_SI_SI,
  SCM_VM_INST_SZ_LSET      = SCM_VM_INST_SZ_SI_SI,
  SCM_VM_INST_SZ_JMP       = SCM_VM_INST_SZ_IOF,
  SCM_VM_INST_SZ_JMPT      = SCM_VM_INST_SZ_IOF,
  SCM_VM_INST_SZ_JMPF      = SCM_VM_INST_SZ_IOF,
  SCM_VM_INST_SZ_LBOX      = SCM_VM_INST_SZ_SI_SI,
  SCM_VM_INST_SZ_CLOSE     = SCM_VM_INST_SZ_SI_SI_OBJ,
  SCM_VM_INST_SZ_DEMINE    = SCM_VM_INST_SZ_SI_SI,
  SCM_VM_INST_SZ_EMINE     = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_EDEMINE   = SCM_VM_INST_SZ_SI_SI,
  SCM_VM_INST_SZ_MRVC      = SCM_VM_INST_SZ_SI,
  SCM_VM_INST_SZ_MRVE      = SCM_VM_INST_SZ_NOOPD,
  SCM_VM_INST_SZ_MODULE    = SCM_VM_INST_SZ_OBJ,
};

extern const int scm_vm_inst_fmt_table[SCM_VM_INST_NR_OP];
extern const size_t scm_vm_inst_fmt_size_table[SCM_VM_INST_NR_FMT];

enum {
  SCM_VM_INST_OFFSET_OP          = (offsetof(struct scm_vm_inst_noopd, op)),
  SCM_VM_INST_OFFSET_OPD         = (offsetof(struct scm_vm_inst_obj, opd)),
  SCM_VM_INST_OFFSET_OBJ_1       = (offsetof(struct scm_vm_inst_obj, opd.o1)),
  SCM_VM_INST_OFFSET_OBJ_OBJ_1   = (offsetof(struct scm_vm_inst_obj_obj, opd.o1)),
  SCM_VM_INST_OFFSET_OBJ_OBJ_2   = (offsetof(struct scm_vm_inst_obj_obj, opd.o2)),
  SCM_VM_INST_OFFSET_SI_1        = (offsetof(struct scm_vm_inst_si, opd.o1)),
  SCM_VM_INST_OFFSET_SI_SI_1     = (offsetof(struct scm_vm_inst_si_si, opd.o1)),
  SCM_VM_INST_OFFSET_SI_SI_2     = (offsetof(struct scm_vm_inst_si_si, opd.o2)),
  SCM_VM_INST_OFFSET_SI_SI_OBJ_1 = (offsetof(struct scm_vm_inst_si_si_obj, opd.o1)),
  SCM_VM_INST_OFFSET_SI_SI_OBJ_2 = (offsetof(struct scm_vm_inst_si_si_obj, opd.o2)),
  SCM_VM_INST_OFFSET_SI_SI_OBJ_3 = (offsetof(struct scm_vm_inst_si_si_obj, opd.o3)),
  SCM_VM_INST_OFFSET_IOF_1       = (offsetof(struct scm_vm_inst_iof, opd.o1)),
};

enum {
  SCM_VM_INST_SZ_OPD_NOOPD     = (SCM_VM_INST_SZ_NOOPD     - SCM_VM_INST_OFFSET_OPD),
  SCM_VM_INST_SZ_OPD_OBJ       = (SCM_VM_INST_SZ_OBJ       - SCM_VM_INST_OFFSET_OPD),
  SCM_VM_INST_SZ_OPD_OBJ_OBJ   = (SCM_VM_INST_SZ_OBJ_OBJ   - SCM_VM_INST_OFFSET_OPD),
  SCM_VM_INST_SZ_OPD_SI        = (SCM_VM_INST_SZ_SI        - SCM_VM_INST_OFFSET_OPD),
  SCM_VM_INST_SZ_OPD_SI_SI     = (SCM_VM_INST_SZ_SI_SI     - SCM_VM_INST_OFFSET_OPD),
  SCM_VM_INST_SZ_OPD_SI_SI_OBJ = (SCM_VM_INST_SZ_SI_SI_OBJ - SCM_VM_INST_OFFSET_OPD),
  SCM_VM_INST_SZ_OPD_IOF       = (SCM_VM_INST_SZ_IOF       - SCM_VM_INST_OFFSET_OPD),
};

static inline int
scm_vm_inst_op2fmt(scm_opcode_t op)
{
  scm_assert(/* 0 <= op &&  */op < SCM_VM_INST_NR_OP);
  return scm_vm_inst_fmt_table[op];
}

static inline size_t
scm_vm_inst_fmt2size(int fmt)
{
  scm_assert(0 <= fmt && fmt < SCM_VM_INST_NR_FMT);
  return scm_vm_inst_fmt_size_table[fmt];
}

static inline size_t
scm_vm_inst_op2size(scm_opcode_t op)
{
  scm_assert(/* 0 <= op &&  */op < SCM_VM_INST_NR_OP);
  return scm_vm_inst_fmt_size_table[scm_vm_inst_fmt_table[op]];
}

static inline size_t
scm_vm_inst_op2opdsize(scm_opcode_t op)
{
  return (scm_vm_inst_op2size(op) - SCM_VM_INST_OFFSET_OPD);
}


static inline scm_opcode_t
scm_vm_inst_get_op(scm_byte_t *ip)
{
  return *SCM_CAST(scm_opcode_t *, ip);
}

static inline scm_opcode_t
scm_vm_inst_fetch_op(scm_byte_t **ip)
{
  scm_opcode_t op = scm_vm_inst_get_op(*ip);
  *ip += SCM_VM_INST_OFFSET_OPD;
  return op;
}

static inline void
scm_vm_inst_fetch_opd_noopd(scm_byte_t **ip)
{
  *ip += SCM_VM_INST_SZ_OPD_NOOPD;
}

static inline void
scm_vm_inst_fetch_opd_obj(scm_byte_t **ip, ScmObj *o)
{
  *o = SCM_CAST(struct scm_vm_inst_opd_obj *, *ip)->o1;
  *ip += SCM_VM_INST_SZ_OPD_OBJ;
}

static inline void
scm_vm_inst_fetch_opd_obj_obj(scm_byte_t **ip, ScmObj *o1, ScmObj *o2)
{
  *o1 = SCM_CAST(struct scm_vm_inst_opd_obj_obj *, *ip)->o1;
  *o2 = SCM_CAST(struct scm_vm_inst_opd_obj_obj *, *ip)->o2;
  *ip += SCM_VM_INST_SZ_OPD_OBJ_OBJ;
}

static inline void
scm_vm_inst_fetch_opd_si(scm_byte_t **ip, int *o)
{
  *o = SCM_CAST(struct scm_vm_inst_opd_si *, *ip)->o1;
  *ip += SCM_VM_INST_SZ_OPD_SI;
}

static inline void
scm_vm_inst_fetch_opd_si_si(scm_byte_t **ip, int *o1, int *o2)
{
  *o1 = SCM_CAST(struct scm_vm_inst_opd_si_si *, *ip)->o1;
  *o2 = SCM_CAST(struct scm_vm_inst_opd_si_si *, *ip)->o2;
  *ip += SCM_VM_INST_SZ_OPD_SI_SI;
}

static inline void
scm_vm_inst_fetch_opd_si_si_obj(scm_byte_t **ip, int *o1, int *o2, ScmObj *o3)
{
  *o1 = SCM_CAST(struct scm_vm_inst_opd_si_si_obj *, *ip)->o1;
  *o2 = SCM_CAST(struct scm_vm_inst_opd_si_si_obj *, *ip)->o2;
  *o3 = SCM_CAST(struct scm_vm_inst_opd_si_si_obj *, *ip)->o3;
  *ip += SCM_VM_INST_SZ_OPD_SI_SI_OBJ;
}

static inline void
scm_vm_inst_fetch_opd_iof(scm_byte_t **ip, int *o)
{
  *o = SCM_CAST(struct scm_vm_inst_opd_iof *, *ip)->o1;
  *ip += SCM_VM_INST_SZ_OPD_IOF;
}


static inline void
scm_vm_inst_set_inst_noopd(scm_byte_t *ip, scm_opcode_t op)
{
  SCM_CAST(struct scm_vm_inst_obj *, ip)->op = op;
}

static inline void
scm_vm_inst_set_inst_obj(scm_byte_t *ip, scm_opcode_t op, ScmObj obj)
{
  SCM_CAST(struct scm_vm_inst_obj *, ip)->op = op;
  SCM_CAST(struct scm_vm_inst_obj *, ip)->opd.o1 = obj;
}

static inline void
scm_vm_inst_set_inst_obj_obj(scm_byte_t *ip,
                            scm_opcode_t op, ScmObj obj1, ScmObj obj2)
{
  SCM_CAST(struct scm_vm_inst_obj_obj *, ip)->op = op;
  SCM_CAST(struct scm_vm_inst_obj_obj *, ip)->opd.o1 = obj1;
  SCM_CAST(struct scm_vm_inst_obj_obj *, ip)->opd.o2 = obj2;
}

static inline void
scm_vm_inst_set_inst_si(scm_byte_t *ip, scm_opcode_t op, int si)
{
  SCM_CAST(struct scm_vm_inst_si *, ip)->op = op;
  SCM_CAST(struct scm_vm_inst_si *, ip)->opd.o1 = si;
}

static inline void
scm_vm_inst_set_inst_si_si(scm_byte_t *ip, scm_opcode_t op, int si1, int si2)
{
  SCM_CAST(struct scm_vm_inst_si_si *, ip)->op = op;
  SCM_CAST(struct scm_vm_inst_si_si *, ip)->opd.o1 = si1;
  SCM_CAST(struct scm_vm_inst_si_si *, ip)->opd.o2 = si2;
}

static inline void
scm_vm_inst_set_inst_si_si_obj(scm_byte_t *ip,
                              scm_opcode_t op, int si1, int si2, ScmObj obj)
{
  SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->op = op;
  SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->opd.o1 = si1;
  SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->opd.o2 = si2;
  SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->opd.o3 = obj;
}

static inline void
scm_vm_inst_set_inst_iof(scm_byte_t *ip, scm_opcode_t op, int iof)
{
  SCM_CAST(struct scm_vm_inst_iof *, ip)->op = op;
  SCM_CAST(struct scm_vm_inst_iof *, ip)->opd.o1 = iof;
}

enum {
  SCM_VM_INST_UPD_FLG_OPD1 = 0x01u,
  SCM_VM_INST_UPD_FLG_OPD2 = 0x02u,
  SCM_VM_INST_UPD_FLG_OPD3 = 0x04u,
};

static inline void
scm_vm_inst_update_opd_obj(scm_byte_t *ip, ScmObj obj,
                          unsigned int flg)
{
  if (flg & SCM_VM_INST_UPD_FLG_OPD1)
    SCM_CAST(struct scm_vm_inst_obj *, ip)->opd.o1 = obj;
}

static inline void
scm_vm_inst_update_opd_obj_obj(scm_byte_t *ip, ScmObj obj1, ScmObj obj2,
                              unsigned int flg)
{
  if (flg & SCM_VM_INST_UPD_FLG_OPD1)
    SCM_CAST(struct scm_vm_inst_obj_obj *, ip)->opd.o1 = obj1;

  if (flg & SCM_VM_INST_UPD_FLG_OPD2)
    SCM_CAST(struct scm_vm_inst_obj_obj *, ip)->opd.o2 = obj2;
}

static inline void
scm_vm_inst_update_opd_si(scm_byte_t *ip, int si, unsigned int flg)
{
  if (flg & SCM_VM_INST_UPD_FLG_OPD1)
    SCM_CAST(struct scm_vm_inst_si *, ip)->opd.o1 = si;
}

static inline void
scm_vm_inst_update_opd_si_si(scm_byte_t *ip, int si1, int si2, unsigned int flg)
{
  if (flg & SCM_VM_INST_UPD_FLG_OPD1)
    SCM_CAST(struct scm_vm_inst_si_si *, ip)->opd.o1 = si1;

  if (flg & SCM_VM_INST_UPD_FLG_OPD2)
    SCM_CAST(struct scm_vm_inst_si_si *, ip)->opd.o2 = si2;
}

static inline void
scm_vm_inst_update_opd_si_si_obj(scm_byte_t *ip, int si1, int si2, ScmObj obj,
                                unsigned int flg)
{
  if (flg & SCM_VM_INST_UPD_FLG_OPD1)
    SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->opd.o1 = si1;

  if (flg & SCM_VM_INST_UPD_FLG_OPD2)
    SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->opd.o2 = si2;

  if (flg & SCM_VM_INST_UPD_FLG_OPD3)
    SCM_CAST(struct scm_vm_inst_si_si_obj *, ip)->opd.o3 = obj;
}

static inline void
scm_vm_inst_update_opd_iof(scm_byte_t *ip, int iof, unsigned int flg)
{
  if (flg & SCM_VM_INST_UPD_FLG_OPD1)
    SCM_CAST(struct scm_vm_inst_iof *, ip)->opd.o1 = iof;
}


#endif /* INCLUDE_VMINST_H__ */
