#include "iseq.c"

#include <sys/types.h>
#include <stddef.h>
#include <stdbool.h>

#include "scythe/vm.h"
#include "scythe/refstk.h"
#include "scythe/module.h"

#include "test.h"

TEST_GROUP(iseq);

static ScmScythe *scy;
static ScmRefStackInfo rsi;
static ScmObj iseq;
static ScmObj iseq2;
static ScmObj iseq3;
static ScmObj iseq4;
static ScmObj mod1;
static ScmObj mod2;

TEST_SETUP(iseq)
{
  scy = ut_scythe_setup(false);
  scm_ref_stack_save(&rsi);

  iseq = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(iseq));
  iseq = scm_iseq_new(SCM_MEM_HEAP);

  iseq2 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(iseq2));
  iseq2 = scm_iseq_new(SCM_MEM_HEAP);

  iseq3 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(iseq3));
  iseq3 = scm_iseq_new(SCM_MEM_HEAP);

  iseq4 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(iseq4));
  iseq4 = scm_iseq_new(SCM_MEM_HEAP);

  mod1 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(mod1));
  mod1 = scm_make_module(ut_read_cstr("(xxx)"), true);

  mod2 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(mod2));
  mod2 = scm_make_module(ut_read_cstr("(yyy)"), true);
}

TEST_TEAR_DOWN(iseq)
{
  scm_ref_stack_restore(&rsi);
  ut_scythe_tear_down(scy);
}

TEST(iseq, iseq_new)
{
  TEST_ASSERT_TRUE(scm_obj_not_null_p(iseq));
  TEST_ASSERT_TRUE(scm_obj_type_p(iseq, &SCM_ISEQ_TYPE_INFO));
  TEST_ASSERT_NOT_NULL(SCM_ISEQ_SEQ_VEC(iseq));
  TEST_ASSERT_NOT_NULL(SCM_ISEQ_OBJS_VEC(iseq));
  TEST_ASSERT_EQUAL_UINT(SCM_ISEQ_DEFAULT_SEQ_SIZE, SCM_ISEQ_SEQ_CAPACITY(iseq));
  TEST_ASSERT_EQUAL_UINT(0, SCM_ISEQ_SEQ_LENGTH(iseq));
  TEST_ASSERT_EQUAL_UINT(SCM_ISEQ_DEFAULT_OBJS_SIZE,
                         SCM_ISEQ_OBJS_CAPACITY(iseq));
  TEST_ASSERT_EQUAL_UINT(0, SCM_ISEQ_OBJS_LENGTH(iseq));
}

static ssize_t
push_inst_noopd(ScmObj iseq, scm_opcode_t op)
{
  struct scm_vm_inst_noopd inst = { .op = scm_internal_opcode(op) };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), NULL, 0);
}

static ssize_t
push_inst_obj(ScmObj iseq, scm_opcode_t op, ScmObj opd1)
{
  struct scm_vm_inst_obj inst = {
    .op = scm_internal_opcode(op), .opd = { .o1 = opd1 }
  };
  size_t objs[1] = { SCM_VM_INST_OFFSET_OBJ_1 };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), objs, 1);
}

static ssize_t
push_inst_obj_obj(ScmObj iseq, scm_opcode_t op, ScmObj opd1, ScmObj opd2)
{
  struct scm_vm_inst_obj_obj inst = {
    .op = scm_internal_opcode(op), .opd = { .o1 = opd1, .o2 = opd2 }
  };
  size_t objs[2] = {
    SCM_VM_INST_OFFSET_OBJ_OBJ_1, SCM_VM_INST_OFFSET_OBJ_OBJ_2
  };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), objs, 2);
}

static ssize_t
push_inst_si(ScmObj iseq, scm_opcode_t op, int opd1)
{
  struct scm_vm_inst_si inst = {
    .op = scm_internal_opcode(op), .opd = { .o1 = opd1 }
  };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), NULL, 0);
}

#if 0

static ssize_t
push_inst_si_si(ScmObj iseq, scm_opcode_t op, int opd1, int opd2)
{
  struct scm_vm_inst_si_si inst = {
    .op = scm_internal_opcode(op), .opd = { .o1 = opd1, .o2 = opd2 }
  };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), NULL, 0);
}

static ssize_t
push_inst_si_si_obj(ScmObj iseq, scm_opcode_t op, int opd1, int opd2, ScmObj opd3)
{
  struct scm_vm_inst_si_si_obj inst = {
    .op = scm_internal_opcode(op), .opd = { .o1 = opd1, .o2 = opd2, .o3 = opd3 }
  };
  size_t objs[1] = { SCM_VM_INST_OFFSET_SI_SI_OBJ_3 };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), objs, 1);
}

static ssize_t
push_inst_iof(ScmObj iseq, scm_opcode_t op, int opd1)
{
  struct scm_vm_inst_iof inst = {
    .op = scm_internal_opcode(op), .opd = { .o1 = opd1 }
  };
  return scm_iseq_push_inst(iseq, &inst, sizeof(inst), NULL, 0);
}

#endif

TEST(iseq, iseq_push_inst2__push)
{
  scm_byte_t *ip;
  scm_opcode_t actual_op;
  int actual_opd;

   TEST_ASSERT_EQUAL_INT(SCM_VM_INST_SZ_EFRAME,
                        push_inst_si(iseq, SCM_OPCODE_EFRAME, 123));

  ip = scm_iseq_to_ip(iseq);
  actual_op = scm_external_opcode(scm_vm_inst_fetch_op(&ip));
  scm_vm_inst_fetch_opd_si(&ip, &actual_opd);

  TEST_ASSERT_EQUAL_INT(SCM_OPCODE_EFRAME, actual_op);
  TEST_ASSERT_EQUAL_INT(123, actual_opd);
}

TEST(iseq, iseq_push_inst2__push_obj)
{
  scm_byte_t *ip;
  ScmObj actual_opd1 = SCM_OBJ_INIT, actual_opd2 = SCM_OBJ_INIT;
  ScmRef ref;
  scm_opcode_t actual_op;

  TEST_ASSERT_EQUAL_INT(SCM_VM_INST_SZ_GREF,
                        push_inst_obj_obj(iseq, SCM_OPCODE_GREF,
                                          SCM_TRUE_OBJ, SCM_FALSE_OBJ));

  ip = scm_iseq_to_ip(iseq);
  actual_op = scm_external_opcode(scm_vm_inst_fetch_op(&ip));
  scm_vm_inst_fetch_opd_obj_obj(&ip, &actual_opd1, &actual_opd2);

  TEST_ASSERT_EQUAL_INT(SCM_OPCODE_GREF, actual_op);
  TEST_ASSERT_SCM_EQ(SCM_TRUE_OBJ, actual_opd1);
  TEST_ASSERT_SCM_EQ(SCM_FALSE_OBJ, actual_opd2);

  TEST_ASSERT_EQUAL_INT(SCM_VM_INST_OFFSET_OBJ_OBJ_1,
                        SCM_ISEQ_OBJS_VEC(iseq)[0]);
  TEST_ASSERT_EQUAL_INT(SCM_VM_INST_OFFSET_OBJ_OBJ_2,
                        SCM_ISEQ_OBJS_VEC(iseq)[1]);

  ref = SCM_REF_MAKE_FROM_PTR((scm_iseq_to_ip(iseq)
                               + SCM_ISEQ_OBJS_VEC(iseq)[0]));
  TEST_ASSERT_SCM_EQ(SCM_TRUE_OBJ, SCM_REF_DEREF(ref));

  ref = SCM_REF_MAKE_FROM_PTR((scm_iseq_to_ip(iseq)
                               + SCM_ISEQ_OBJS_VEC(iseq)[1]));
  TEST_ASSERT_SCM_EQ(SCM_FALSE_OBJ, SCM_REF_DEREF(ref));
}

TEST(iseq, expand_sequence_buffer)
{
  size_t sum;
  ssize_t ret_value;

  for (sum = 0; sum < SCM_ISEQ_DEFAULT_SEQ_SIZE; sum += SCM_VM_INST_SZ_NOP)
    push_inst_noopd(iseq, SCM_OPCODE_NOP);

  ret_value =     push_inst_noopd(iseq, SCM_OPCODE_NOP);

  TEST_ASSERT_EQUAL_INT(sum + SCM_VM_INST_SZ_NOP, ret_value);
  TEST_ASSERT_TRUE(SCM_ISEQ_SEQ_CAPACITY(iseq) > SCM_ISEQ_DEFAULT_SEQ_SIZE);
}

TEST(iseq, expand_index_buffer)
{
  for (size_t sum = 0; sum < SCM_ISEQ_DEFAULT_OBJS_SIZE; sum++)
    push_inst_obj(iseq, SCM_OPCODE_IMMVAL, SCM_NIL_OBJ);

  push_inst_obj(iseq, SCM_OPCODE_IMMVAL, SCM_NIL_OBJ);

  TEST_ASSERT_TRUE(SCM_ISEQ_OBJS_CAPACITY(iseq) > SCM_ISEQ_DEFAULT_OBJS_SIZE);
}
