#include "procedure.c"

#include "scythe/vm.h"
#include "scythe/refstk.h"
#include "scythe/module.h"
#include "scythe/assembler.h"

#include "test.h"

TEST_GROUP(procedure);

static ScmScythe *scy;
static ScmRefStackInfo rsi;
static ScmObj asm1;
static ScmObj asm2;
static ScmObj asm3;
static ScmObj asm4;
static ScmObj mod1;
static ScmObj mod2;

TEST_SETUP(procedure)
{
  ssize_t id;

  scy = ut_scythe_setup(false);
  scm_ref_stack_save(&rsi);

  asm1 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(asm1));
  asm1 = scm_make_assembler(SCM_OBJ_NULL);

  asm2 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(asm2));
  asm2 = scm_make_assembler(SCM_OBJ_NULL);

  asm3 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(asm3));
  id = scm_asm_assign_sequence_id(asm1);
  asm3 = scm_asm_new_seq(asm1, (size_t)id);

  asm4 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(asm4));
  id = scm_asm_assign_sequence_id(asm2);
  asm4 = scm_asm_new_seq(asm2, (size_t)id);

  mod1 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(mod1));
  mod1 = scm_make_module(ut_read_cstr("(xxx)"), true);

  mod2 = SCM_OBJ_NULL;
  scm_register_extra_rfrn(SCM_REF_MAKE(mod2));
  mod2 = scm_make_module(ut_read_cstr("(yyy)"), true);
}

TEST_TEAR_DOWN(procedure)
{
  scm_ref_stack_restore(&rsi);
  ut_scythe_tear_down(scy);
}

static void
test_prog_eq__equal(ScmObj a1, ScmObj a2)
{
  bool actual_rslt;

  scm_asm_commit(a1);
  scm_asm_commit(a2);

  TEST_ASSERT_EQUAL_INT(0,
                        scm_cdprogram_eq(scm_asm_cdprogram(a1),
                                         scm_asm_cdprogram(a2),
                                         &actual_rslt));
  TEST_ASSERT_TRUE(actual_rslt);
}

static void
test_prog_eq__not_equal(ScmObj a1, ScmObj a2)
{
  bool actual_rslt;

  scm_asm_commit(a1);
  scm_asm_commit(a2);

  TEST_ASSERT_EQUAL_INT(0,
                        scm_cdprogram_eq(scm_asm_cdprogram(a1),
                                         scm_asm_cdprogram(a2),
                                         &actual_rslt));
  TEST_ASSERT_FALSE(actual_rslt);
}

TEST(procedure, cdprogram_eq__noopd__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_PUSH);
  scm_asm_push(asm2, SCM_OPCODE_PUSH);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__noopd__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_PUSH);
  scm_asm_push(asm2, SCM_OPCODE_NOP);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__obj__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_IMMVAL, SCM_NIL_OBJ);
  scm_asm_push(asm2, SCM_OPCODE_IMMVAL, SCM_NIL_OBJ);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__obj__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_IMMVAL, SCM_NIL_OBJ);
  scm_asm_push(asm2, SCM_OPCODE_IMMVAL, SCM_EOF_OBJ);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__gv_mod__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_GREF, SCM_NIL_OBJ, mod1);
  scm_asm_push(asm2, SCM_OPCODE_GREF, SCM_NIL_OBJ, mod1);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__obj_mod__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_GREF, SCM_NIL_OBJ, mod1);
  scm_asm_push(asm2, SCM_OPCODE_GREF, SCM_NIL_OBJ, mod2);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_CALL, 1);
  scm_asm_push(asm2, SCM_OPCODE_CALL, 1);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_CALL, 1);
  scm_asm_push(asm2, SCM_OPCODE_CALL, 10);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si_si__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_LREF, 1, 1);
  scm_asm_push(asm2, SCM_OPCODE_LREF, 1, 1);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si_si__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_LREF, 1, 1);
  scm_asm_push(asm2, SCM_OPCODE_LREF, 1, -20);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si_si_seq__equal)
{
  scm_asm_push(asm3, SCM_OPCODE_NOP);
  scm_asm_push(asm4, SCM_OPCODE_NOP);

  scm_asm_push(asm1, SCM_OPCODE_CLOSE, 1, 1, asm3);
  scm_asm_push(asm2, SCM_OPCODE_CLOSE, 1, 1, asm4);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si_si_seq__not_equal)
{
  scm_asm_push(asm3, SCM_OPCODE_NOP);
  scm_asm_push(asm4, SCM_OPCODE_NOP);

  scm_asm_push(asm1, SCM_OPCODE_CLOSE, 1, 1, asm3);
  scm_asm_push(asm2, SCM_OPCODE_CLOSE, 1, 2, asm4);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__si_si_seq__not_equal__inner_asm)
{
  scm_asm_push(asm3, SCM_OPCODE_NOP);
  scm_asm_push(asm4, SCM_OPCODE_PUSH);

  scm_asm_push(asm1, SCM_OPCODE_CLOSE, 1, 1, asm3);
  scm_asm_push(asm2, SCM_OPCODE_CLOSE, 1, 1, asm4);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__iof__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_JMP, false, -(int)SCM_VM_INST_SZ_IOF);
  scm_asm_push(asm2, SCM_OPCODE_JMP, false, -(int)SCM_VM_INST_SZ_IOF);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__iof__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_JMP, false, -(int)SCM_VM_INST_SZ_IOF);
  scm_asm_push(asm2, SCM_OPCODE_JMP, false, 0);
  test_prog_eq__not_equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__mod__equal)
{
  scm_asm_push(asm1, SCM_OPCODE_MODULE, mod1);
  scm_asm_push(asm2, SCM_OPCODE_MODULE, mod1);
  test_prog_eq__equal(asm1, asm2);
}

TEST(procedure, cdprogram_eq__mod__not_equal)
{
  scm_asm_push(asm1, SCM_OPCODE_MODULE, mod1);
  scm_asm_push(asm2, SCM_OPCODE_MODULE, mod2);
  test_prog_eq__not_equal(asm1, asm2);
}
